# coding=utf-8

"""
Pyccuweather
The Python Accuweather API

(c) Chris von Csefalvay, 2015.

http://www.github.com/chrisvoncsefalvay/pyccuweather/
"""


from collections import OrderedDict
from time import strptime
from uuid import uuid4


class Region(object):
    """
    An object representing an Accuweather Region.
    For a list of Accuweather regions: http://apidev.accuweather.com/developers/regions
    """
    def __init__(self, json=None, identifier=None, localized_name=None, english_name=None):

        assert json or (localized_name and english_name and identifier)

        if json:
            identifier = json["ID"]
            localized_name = json["LocalizedName"]
            english_name = json["EnglishName"]

        self.id = identifier
        self.localized_name = localized_name
        self.english_name = english_name

    def __str__(self):
        return u"<Region {0:s} ({1:s})>".format(self.english_name, self.id)


class AdministrativeArea(object):
    """
    Represents a primary administrative area.
    Countries are explained in the Accuweather documentation at http://apidev.accuweather.com/developers/administrativeareas
    """
    def __init__(self,
                 json=None,
                 identifier=None,
                 localized_name=None,
                 english_name=None,
                 level=None,
                 localized_type=None,
                 english_type=None):
        assert json or (identifier and localized_name and english_name and level and localized_type and english_type)

        if json:
            identifier = json["ID"]
            localized_name = json["LocalizedName"]
            english_name = json["EnglishName"]
            level = json["Level"]
            localized_type = json["LocalizedType"]
            english_type = json["EnglishType"]

        self.id = identifier
        self.localized_name = localized_name
        self.english_name = english_name
        self.level = level
        self.localized_type = localized_type
        self.english_type = english_type

    def __str__(self):
        return u"<{0:s} {1:s} ({2:s})>".format(self.english_type, self.english_name, self.id)


class Country(object):
    """
    Represents a Country.
    Countries are explained in the Accuweather documentation at http://apidev.accuweather.com/developers/countries
    """
    def __init__(self, json=None, identifier=None, localized_name=None, english_name=None):
        assert json or (localized_name and english_name and identifier)

        if json:
            identifier = json["ID"]
            localized_name = json["LocalizedName"]
            english_name = json["EnglishName"]

        self.id = identifier
        self.localized_name = localized_name
        self.english_name = english_name

    def __str__(self):
        return u"<Country: {0:s} ({1:s})>".format(self.english_name, self.id)


class TimeZone(object):
    """
    Represents a timezone.
    Timezones are explained in the Accuweather documentation at http://apidev.accuweather.com/developers/timeZones
    """
    def __init__(self,
                 json=None,
                 code=None,
                 name=None,
                 gmt_offset=None,
                 is_daylight_saving=None,
                 next_offset_change=None):
        assert json or (code and name and gmt_offset and is_daylight_saving and next_offset_change)

        if not json:
            assert isinstance(is_daylight_saving, bool)
            assert isinstance(gmt_offset, (float and int))
            assert isinstance(next_offset_change, str) and len(next_offset_change) == 20

        if json:
            code = json["Code"]
            name = json["Name"]
            gmt_offset = json["GmtOffset"]
            is_daylight_saving = True if json["IsDaylightSaving"] is 'true' else False
            next_offset_change = json["NextOffsetChange"]

        self.code = code
        self.name = name
        self.gmt_offset = gmt_offset
        self.is_daylight_saving = is_daylight_saving
        self.next_offset_change = strptime(next_offset_change, "%Y-%m-%dT%H:%M:%SZ")


class Location(object):
    """
    Represents a Location.
    Locations are explained in the Accuweather documentation at http://apidev.accuweather.com/developers/locations
    """
    def __init__(self,
                 json = None,
                 lkey = None,
                 lat: float=None,
                 lon: float=None,
                 localized_name: str=None,
                 english_name: str=None,
                 region: Region=None,
                 country: Country=None,
                 administrative_area: AdministrativeArea=None,
                 timezone: TimeZone=None):

        if json:
            lkey = json["Key"]
            lat = json["GeoPosition"]["Latitude"]
            lon = json["GeoPosition"]["Longitude"]
            localized_name = json["LocalizedName"]
            english_name = json["EnglishName"]
            region = json["Region"]
            country = json["Country"]
            administrative_area = json["AdministrativeArea"]
            timezone = json["TimeZone"]

        self.lkey = lkey
        self.lat = lat
        self.lon = lon
        self.localized_name = localized_name
        self.english_name = english_name
        self.region = Region(json=region)
        self.country = Country(json=country)
        self.administrative_area = AdministrativeArea(json=administrative_area)
        self.timezone = TimeZone(json=timezone)

    def __str__(self):
        return u"<Location key: {0} ({1})>".format(self.lkey, self.english_name)

    __repr__ = __str__


class LocationSet(object):
    """
    Represents a set of Locations returned usually as a search result.
    """
    def __init__(self, results: list, search_expression: str, country: str=None):
        self.results = results
        self.search_expression = search_expression
        self.country = country

    def __len__(self):
        return len(self.results)

    def __getitem__(self, item):
        return self.results[item]

    def __str__(self):
        return u"<Location result set for the query '{0:s}' ({1:d} results)>".format(self.search_expression,
                                                                                     len(self.results))

    __repr__ = __str__

    def append(self, location:Location):
        """
        Appends a location to the location result set.

        :param location: Location object
        :return: void
        """
        self.results.append(location)


class Temperature(object):
    """
    Represents a temperature value.
    """
    def __init__(self, value, units="C"):
        assert units in ["C", "F"]

        self.value = value
        self.units = units

    @property
    def F(self):
        """
        Represents the Temperature in Fahrenheit.

        :return: Temperature in degrees Fahrenheit.
        """
        if self.units == "F":
            return self.value
        else:
            return (self.value * 1.8) + 32

    @property
    def C(self):
        """
        Represents the Temperature in Celsius.

        :return: Temperature in degrees Celsius.
        """
        if self.units == "C":
            return self.value
        else:
            return (self.value - 32) * (5 / 9)

    def __str__(self):
        return u"<Temperature: {0:.1f} C / {1:.1f} F>".format(self.C, self.F)

    __repr__ = __str__


class Precipitation(object):
    """
    Represents a precipitation value.
    """
    def __init__(self, value, units="mm"):
        assert units in ["mm", "in"]

        self.value = value
        self.units = units

    @property
    def mm(self):
        """
        Represents precipitation in mm.

        :return: precipitation in mm
        """
        if self.units == "mm":
            return self.value
        else:
            return self.value * 25.4

    @property
    def inch(self):
        """
        Represents precipitation in inches.

        :return: precipitation in in
        """
        if self.units == "in":
            return self.value
        else:
            return self.value / 25.4

    def __str__(self):
        return u"<Precipitation: {0:.1f} mm / {1:.1f} in>".format(self.mm, self.inch)

    __repr__ = __str__


class Snow(object):
    def __init__(self, value, units="cm"):
        assert units in ["cm", "in"]

        self.value = value
        self.units = units

    @property
    def mm(self):
        if self.units == "cm":
            return 10 * self.value
        else:
            return self.value * 25.4

    def inch(self):
        if self.units == "in":
            return self.value
        else:
            return self.value / 2.54

    def __str__(self):
        return u"<Snow: {0:.1f} mm / {1:.1f} in>".format(self.mm, self.inch)

    __repr__ = __str__


class Wind(object):
    def __init__(self, json, hdg=None):

        self.speed = json["Speed"]["Value"]
        self.units = json["Speed"]["Unit"]
        if hdg is not None:
            self.hdg = hdg
        else:
            self.hdg = json["Direction"]["Degrees"]

    def kmh(self):
        if self.units == "km/h":
            return self.speed
        else:
            return self.speed * 1.60934

    def mph(self):
        if self.units == "mi/h":
            return self.speed
        else:
            return self.speed / 1.60934

    def __str__(self):
        return u"<Wind: {0} {1:.1f} kmh / {2:.1f} mph>".format(self.hdg, self.kmh, self.mph)

    __repr__ = __str__


class AirQualityFactor(object):
    def __init__(self, aqf_dict):
        self.name = aqf_dict["Name"]
        self.value = aqf_dict["Value"]
        self.category = aqf_dict["Category"]
        self.band = aqf_dict["CategoryValue"]


    def __str__(self):
        return u"<Air quality factor {0:s}: {1:.2f} ({2:s})>".format(self.name, self.value, self.band)

    __repr__ = __str__


class AirQuality(object):
    def __init__(self, aqf_dict):
        for each in aqf_dict:
            aqi = AirQualityFactor(each)
            exec("self.%s = aqi" % each["Name"].lower())


class Ceiling(object):
    def __init__(self, json):
        self.value = json["Value"]
        self.units = json["Unit"]

    @property
    def km(self):
        if self.units == "km":
            return self.value
        else:
            return self.value * 0.0003048

    @property
    def m(self):
        if self.units == "km":
            return self.value * 1000
        else:
            return self.value * 0.3048

    @property
    def ft(self):
        if self.units == "ft":
            return self.value
        else:
            return self.value * 3280.8399

    def __str__(self):
        return u"<Ceiling at {0:s}m (approximately FL {1:s})>".format(self.m, int(round(self.ft / 100, -1)))

    __repr__ = __str__

class Hemiurnal(object):
    def __init__(self, json):
        self.id = uuid4()
        # Verbals
        self.synopsis = json["LongPhrase"]
        self.phrase = json["ShortPhrase"]
        # Precipitation
        self.snow = Snow(value=json["Snow"]["Value"],
                         units=json["Snow"]["Unit"])
        self.wind = Wind(json["Wind"])
        self.rain = Precipitation(value=json["Rain"]["Value"],
                                  units=json["Rain"]["Unit"])
        self.ice = Precipitation(value=json["Ice"]["Value"],
                                 units=json["Ice"]["Unit"])
        self.total_liquid = Precipitation(value=json["Ice"]["Value"],
                                          units=json["Ice"]["Unit"])
        self.h_precipitation = json["HoursOfPrecipitation"]
        self.h_rain = json["HoursOfRain"]
        # Cloud cover
        self.cloud_cover = json["CloudCover"]
        # Probabilities
        self.p_rain = json["RainProbability"]
        self.p_snow = json["SnowProbability"]
        self.p_ice = json["IceProbability"]
        self.p_thunderstorm = json["ThunderstormProbability"]
        self.p_precipitation = json["PrecipitationProbability"]
        # Wind
        if "WindGust" in json.keys():
            self.wind_gust = Wind(json["WindGust"])
        else:
            self.wind_gust = None
        self.raw = json

    def __str__(self):
        return u"<Hemiurnal observation {0:s}>".format(self.id)

class DegreeDay(object):
    def __init__(self, aqf_dict):
        self.cooling = Temperature(value=aqf_dict["Cooling"]["Value"], units=aqf_dict["Cooling"]["Unit"])
        self.warming = Temperature(value=aqf_dict["Warming"]["Value"], units=aqf_dict["Warming"]["Unit"])


class DailyForecast(object):
    def __init__(self, json):
        # Dates
        self.epoch_date = json["EpochDate"]
        self.date = json["Date"]
        # Temperatures
        self.temp_min = Temperature(value=json["Temperature"]["Minimum"]["Value"],
                                    units=json["Temperature"]["Minimum"]["Unit"])
        self.temp_max = Temperature(value=json["Temperature"]["Maximum"]["Value"],
                                    units=json["Temperature"]["Maximum"]["Unit"])
        self.realfeel_temp_min = Temperature(value=json["RealFeelTemperature"]["Minimum"]["Value"],
                                             units=json["RealFeelTemperature"]["Minimum"]["Unit"])
        self.realfeel_temp_max = Temperature(value=json["RealFeelTemperature"]["Maximum"]["Value"],
                                             units=json["RealFeelTemperature"]["Maximum"]["Unit"])
        self.realfeel_shade_temp_min = Temperature(value=json["RealFeelTemperatureShade"]["Minimum"]["Value"],
                                                   units=json["RealFeelTemperatureShade"]["Minimum"]["Unit"])
        self.realfeel_shade_temp_max = Temperature(value=json["RealFeelTemperatureShade"]["Maximum"]["Value"],
                                                   units=json["RealFeelTemperatureShade"]["Maximum"]["Unit"])
        # Sunshine hours
        self.hours_of_sun = json["HoursOfSun"]
        # Hemiurnals
        self.day = Hemiurnal(json["Day"])
        self.night = Hemiurnal(json["Night"])
        self.raw = json

    def __str__(self):
        return u"<Daily forecast for {0:s}>".format(self.date)

    __repr__ = __str__

class HourlyForecast(object):
    def __init__(self, json):

        # Dates and times
        self.epoch_datetime = json["EpochDateTime"]
        self.datetime = json["DateTime"]
        # Temperatures
        self.temperature = Temperature(value=json["Temperature"]["Value"],
                                       units=json["Temperature"]["Unit"])
        self.realfeel_temperature = Temperature(value=json["Temperature"]["Value"],
                                                units=json["Temperature"]["Unit"])
        # Cloud cover and ceiling
        self.cloud_cover = json["CloudCover"]
        self.ceiling = Ceiling(json["Ceiling"])
        # Wind
        self.wind = Wind(json["Wind"])

        if "WindGust" in json.keys():
            if "Direction" in json["WindGust"].keys():
                self.wind_gust = Wind(json["WindGust"])
            else:
                self.wind_gust = Wind(json["WindGust"], hdg=self.wind.hdg)
        else:
            self.wind_gust = None
        # rH% and dew point
        self.rh = json["RelativeHumidity"]
        self.dewpoint = Temperature(value=json["DewPoint"]["Value"],
                                    units=json["DewPoint"]["Unit"])
        self.wet_bulb_temperature = Temperature(value=json["WetBulbTemperature"]["Value"],
                                                units=json["WetBulbTemperature"]["Unit"])
        # UV index
        self.uv_index = json["UVIndex"]
        self.uv_index_text = json["UVIndexText"]
        # Precipitation
        self.rain = Precipitation(value=json["Rain"]["Value"],
                                  units=json["Rain"]["Unit"])
        self.total_liquid = Precipitation(value=json["TotalLiquid"]["Value"],
                                          units=json["TotalLiquid"]["Unit"])
        self.ice = Precipitation(value=json["Ice"]["Value"],
                                 units=json["Ice"]["Unit"])
        self.snow = Snow(value=json["Snow"]["Value"],
                         units=json["Snow"]["Unit"])
        # Probabilities
        self.p_snow = json["SnowProbability"]
        self.p_ice = json["IceProbability"]
        self.p_rain = json["RainProbability"]
        self.p_precipitation = json["PrecipitationProbability"]
        # Accuweather
        self.link = json["Link"]
        self.mobile_link = json["MobileLink"]
        self.raw = json

    def __str__(self):
        return u"<Hourly forecast for {0:s}>".format(self.datetime)

    __repr__ = __str__



class DailyForecasts(object):
    def __init__(self, json):
        self.effective_date = json["Headline"]["EffectiveDate"]
        self.effective_epoch_date = json["Headline"]["EffectiveEpochDate"]
        self.end_date = json["Headline"]["EndDate"]
        self.end_epoch_date = json["Headline"]["EndEpochDate"]
        self.severity = json["Headline"]["Severity"]
        self.synopsis = json["Headline"]["Text"]
        self.link = json["Headline"]["Link"]
        self.mobile_link = json["Headline"]["MobileLink"]

        self.forecasts = OrderedDict()
        for each in json["DailyForecasts"]:
            k = each["Date"][0:10]
            v = DailyForecast(each)
            self.forecasts[k] = v
        self.raw = json

    def __str__(self):
        return "<Daily forecasts from %s to %s>" % (self.effective_date, self.end_date)

    __repr__ = __str__

class HourlyForecasts(object):
    def __init__(self, json):
        self.forecasts = OrderedDict()
        for each in json:
            k = each["DateTime"]
            v = HourlyForecast(each)
            self.forecasts[k] = v
        self.raw = json

    def __str__(self):
        return "<Hourly forecasts from %s>" % self.forecasts[0]

    __repr__ = __str__


class Observation(object):
    def __init__(self, json):
        # Date and time
        self.date_time = json["LocalObservationDateTime"]
        self.epoch_time = json["EpochTime"]
        self.synopsis = json["WeatherText"]
        self.temperature = Temperature(value=json["Temperature"]["Metric"]["Value"],
                                       units=json["Temperature"]["Metric"]["Unit"])
        self.link = json["Link"]
        self.mobile_link = json["MobileLink"]
        self.raw = json

    def __str__(self):
        return "<Synoptic observation at %s>" % self.date_time

    __repr__ = __str__



class CurrentObs(object):
    def __init__(self, json):
        self.observations = OrderedDict()
        for each in json:
            k = each["LocalObservationDateTime"]
            v = each
            self.observations[k] = v
        self.raw = json

    def __str__(self):
        return "<Current observations from %s>" % self.observations[0]
    __repr__ = __str__



# coding=utf-8

"""
Pyccuweather
The Python Accuweather API

connector.py
Basic connector object and methods

(c) Chris von Csefalvay, 2015.
"""

import requests
from pyccuweather import errors
from pyccuweather.froots import froot
from pyccuweather.objects import *
import os


class Connection(object):
    """
    Represents a connection to the Accuweather API.

    :param API_KEY: API key
    :param dev: whether the dev mode api (apidev.accuweather.com) or the production api (api.accuweather.com) is used
    :param retry: number of retries of failed operations - TODO: implement
    :raise errors.MalformattedAPIKeyError: if the API key is not a 32-character string, an error is thrown
    """

    def __init__(self, API_KEY: str=None, dev: bool=True, retry: int=3):

        # TODO: implement retries

        try:
            self.API_KEY = os.environ["ACCUWEATHER_APIKEY"]
        except KeyError:
            try:
                assert isinstance(API_KEY, str)
                assert len(API_KEY) is 32
                self.API_KEY = API_KEY
            except AssertionError:
                raise errors.MalformattedAPIKeyError()

        self.API_ROOT = "http://apidev.accuweather.com" if dev is True else "http://api.accuweather.com"
        self.API_VERSION = "v1"
        self.retries = retry

    def __str__(self):
        return u"Accuweather connector to {0:s}".format(self.API_ROOT)

    def wipe_api_key(self):
        """
        Wipes API key from a Connection instance
        :return: void
        """
        self.API_KEY = None

    ########################################################
    # Location resolvers                                   #
    ########################################################

    def loc_geoposition(self, lat: float, lon: float):
        """
        Resolves location based on geoposition.

        :param lat: latitude
        :param lon: longitude
        :return: Location object
        """

        try:
            assert isinstance(lat, (int, float)) and isinstance(lon, (int, float))
        except:
            raise ValueError

        try:
            assert abs(lat) <= 90 and abs(lon) <= 180
        except:
            raise errors.RangeError(lat, lon)

        payload = {"q": u"{0:.4f},{1:.4f}".format(lat, lon),
                   "apikey": self.API_KEY}

        resp = requests.get(url=froot("loc_geoposition"),
                            params=payload).json()

        assert len(resp) > 0

        if isinstance(resp, list):
            return Location(resp[0])
        elif isinstance(resp, dict):
            return Location(resp)

    def loc_string(self, search_string: str, country_code: str=None):
        """
        Resolves a search string and an optional country code to a location.

        :param search_string: search string
        :param country_code: country code to which the search will be limited
        :return: a LocationSet of results
        """

        if country_code is not None:
            try:
                assert len(country_code) is 2
            except:
                raise errors.InvalidCountryCodeError(country_code)

            url = froot("loc_search_country", country_code=country_code)
            payload = {"q": search_string,
                       "apikey": self.API_KEY}

        else:
            url = froot("loc_search")
            payload = {"q": search_string,
                       "apikey": self.API_KEY}

        resp = requests.get(url=url,
                            params=payload).json()

        _result = list()
        if len(resp) > 0:
            for each in resp:
                loc = Location(lkey=each["Key"],
                               lat=each["GeoPosition"]["Latitude"],
                               lon=each["GeoPosition"]["Longitude"],
                               localized_name=each["LocalizedName"],
                               english_name=each["EnglishName"],
                               region=each["Region"],
                               country=each["Country"],
                               administrative_area=each["AdministrativeArea"],
                               timezone=each["TimeZone"]
                               )
                _result.append(loc)
        else:
            raise errors.NoResultsError(search_string)

        return (LocationSet(results=_result,
                            search_expression=search_string,
                            country=country_code))

    def loc_postcode(self, country_code: str, postcode: str):
        """
        Resolves location based on postcode. Only works in selected countries (US, Canada).

        :param country_code: Two-letter country code
        :param postcode: Postcode
        :return: Location object
        """

        try:
            assert len(country_code) is 2
        except:
            raise errors.InvalidCountryCodeError(country_code)

        url = froot("loc_postcode", country_code=country_code)
        payload = {"q": postcode,
                   "apikey": self.API_KEY}

        resp = requests.get(url=url,
                            params=payload).json()

        assert len(resp) > 0

        if isinstance(resp, list):
            return Location(resp[0])
        elif isinstance(resp, dict):
            return Location(resp)

    def loc_ip(self, ip_address:str):
        """
        Resolves location based on IP address.

        :param ip_address: IP address
        :return: Location object
        """

        url = froot("loc_ip_address")
        payload = {"q": ip_address,
                   "apikey": self.API_KEY}

        resp = requests.get(url=url,
                            params=payload).json()

        assert len(resp) > 0

        if isinstance(resp, list):
            return Location(resp[0])
        elif isinstance(resp, dict):
            return Location(resp)

    def loc_lkey(self, lkey:int):
        """
        Resolves location by Accuweather location key.

        :param lkey: Accuweather location key
        :return: Location object
        """

        assert isinstance(lkey, int)

        url = froot("loc_lkey", location_key=lkey)
        payload = {"apikey": self.API_KEY}

        resp = requests.get(url=url,
                            params=payload).json()

        assert len(resp) > 0

        if isinstance(resp, list):
            return Location(resp[0])
        elif isinstance(resp, dict):
            return Location(resp)

    ########################################################
    # Current conditions                                   #
    ########################################################

    def get_current_wx(self, lkey:int=None, location:Location=None, current:int=0, details:bool=True):
        """
        Get current weather conditions.

        :param lkey: Accuweather location key
        :param location: Location object
        :param current: horizon - current weather, 6 hours or 24 hours
        :param details: should details be provided?
        :return: raw observations or CurrentObs object
        """

        assert current in [0, 6, 24]
        assert lkey is not None or location is not None
        if current is 0:
            url = froot("currentconditions", location_key=lkey)
        else:
            url = froot("currentconditions_{current}".format(current=current), location_key=lkey)

        payload = {"apikey": self.API_KEY,
                   "details": "true" if details is True else "false"}

        resp = requests.get(url=url,
                            params=payload)

        return CurrentObs(resp.json())

    ########################################################
    # Forecasts                                            #
    ########################################################

    def get_forecast(self, forecast_type:str, lkey:int, details:bool=True, metric:bool=True):
        forecast_types = ["1h", "12h", "24h", "72h", "120h", "240h",
                          "1d", "5d", "10d", "15d", "25d", "45d"]
        assert forecast_type in forecast_types

        fkeyid = u"forecast_{0:s}".format(forecast_type)

        url = froot(fkeyid, location_key=lkey)
        payload = {"apikey": self.API_KEY,
                   "details": "true" if details == True else "false",
                   "metric": "true" if metric == True else "false"}

        resp = requests.get(url=url,
                            params=payload)

        if forecast_type[-1] is "h":
            return HourlyForecasts(resp.json())
        elif forecast_type[-1] is "d":
            return DailyForecasts(resp.json())

    ########################################################
    # Air quality                                          #
    ########################################################

    def get_airquality(self, lkey:int, current:bool=True):
        assert isinstance(lkey, int)

        if current:
            fkeyid = "airquality_current"
        else:
            fkeyid = "airquality_yesterday"

        url = froot(fkeyid, location_key=lkey)
        payload = {"apikey": self.API_KEY}

        return requests.get(url=url,
                            params=payload)

    ########################################################
    # Climo                                                #
    ########################################################

    def get_actuals(self, lkey:int, start_date:str, end_date:str=None):

        # TODO: Return object
        # (needs API access)

        if end_date:
            fkeyid = "climo_actuals_range"
            url = froot(fkeyid, location_key=lkey)
            payload = {"apikey": self.API_KEY,
                       "start": start_date,
                       "end": end_date}
        else:
            fkeyid = "climo_actuals_date"
            url = froot(fkeyid,
                        date=start_date,
                        location_key=lkey)
            payload = {"apikey": self.API_KEY}

        return requests.get(url=url,
                            params=payload)

    def get_records(self, lkey, start_date, end_date=None):

        # TODO: Return object
        # (needs API access)

        if end_date:
            fkeyid = "climo_records_range"
            url = froot(fkeyid, location_key=lkey)
            payload = {"apikey": self.API_KEY,
                       "start": start_date,
                       "end": end_date}
        else:
            fkeyid = "climo_records_date"
            url = froot(fkeyid,
                        date=start_date,
                        location_key=lkey)
            payload = {"apikey": self.API_KEY}

        return requests.get(url=url,
                            params=payload)

    def get_normals(self, lkey, start_date, end_date=None):

        # TODO: Return object
        # (needs API access)

        if end_date:
            fkeyid = "climo_normals_range"
            url = froot(fkeyid, location_key=lkey)
            payload = {"apikey": self.API_KEY,
                       "start": start_date,
                       "end": end_date}
        else:
            fkeyid = "climo_normals_date"
            url = froot(fkeyid,
                        date=start_date,
                        location_key=lkey)
            payload = {"apikey": self.API_KEY}

        return requests.get(url=url,
                            params=payload)

    ########################################################
    # Alerts                                               #
    ########################################################

    def get_alerts(self, lkey, forecast_range):

        # TODO: Return object
        # (needs API access)

        assert isinstance(forecast_range, int)
        fkeyid = u"alarms_{0:d}d".format(forecast_range)
        url = froot(fkeyid, location_key=lkey)
        payload = {"apikey": self.API_KEY}

        return requests.get(url=url,
                            params=payload)


# coding=utf-8

"""
Pyccuweather
The Python Accuweather API

(c) Chris von Csefalvay, 2015.

http://www.github.com/chrisvoncsefalvay/pyccuweather/
"""

import json
from time import gmtime
from datetime import date


def wloads(content):
    """
    Decodes incoming JSON with UTF-8.

    :param content: JSON formatted content
    :return: JSON formatted content loaded as object and decoded with UTF-8.
    """
    return json.loads(content.decode('utf-8'))


def get_woy(epochdate):
    """
    Converts an epoch date into week of year.

    :param epochdate: Epoch date
    :return: week of year
    """
    _time = gmtime(epochdate)
    return date(_time.tm_year, _time.tm_mon, _time.tm_mday).isocalendar()[1]



# coding=utf-8

"""
Pyccuweather
The Python Accuweather API

(c) Chris von Csefalvay, 2015.

http://www.github.com/chrisvoncsefalvay/pyccuweather/
"""


class MalformattedAPIKeyError(BaseException):
    """
    Raised when the API key looks malformatted (wrong type or length). Does not actually check the API key for validity!"
    """
    def __str__(self):
        return "Malformatted API key: your API key must be a 32-character hexadecimal string."


class RangeError(BaseException):
    """
    Raised when the coordinates provided are out of range.
    """
    def __str__(self):
        return "Coordinates out of range."


class NoLocationError(BaseException):
    """
    Raised when the coordinates do not resolve to a particular location.
    """
    def __init__(self, lat, lon):
        self.lat = lat
        self.lon = lon

    def __str__(self):
        return u"The coordinates {0:s} do not resolve to a correct location.".format(self.lat, self.lon)


class UnauthorisedError(BaseException):
    """
    Raised when the connection was not authorised.
    """
    def __str__(self):
        return "The server returned a HTTP1.1 403/Forbidden response."


class APIConnectionError(BaseException):
    """
    Raised when no connection to the Accuweather servers could be established.
    """
    def __str__(self):
        return u"Could not connect. You can check your connection by pinging Accuweather."


class APIError(BaseException):
    """
    Raised when the Accuweather servers returned a HTTP error code.
    """
    def __init__(self, status_code):
        self.status_code = status_code

    def __str__(self):
        return u"The Accuweather API returned an error: {0:s}".format(self.status_code)


class NotImplementedOrUnknownMethod(BaseException):
    """
    The method called has not been implemented yet or is an altogether non-existent method.
    """
    def __init__(self, API_method):
        self.API_method = API_method

    def __str__(self):
        return (u"The {0:s} method is not implemented in this client. "
                u"There's a chance this is because it does not exist.".format(self.API_method))


class InvalidCountryCodeError(BaseException):
    """
    Raised when a country code is provided that does not exist.
    Valid country codes are listed by Accuweather under http://apidev.accuweather.com/developers/countries
    """
    def __init__(self, country_code):
        self.country_code = country_code

    def __str__(self):
        return(u"The country code {0:s} could not be resolved.\n"
               u"For a list of valid two-letter country codes, please go to:\n"
               u"http://apidev.accuweather.com/developers/countries".format(self.country_code))


class MalformattedLocationKeyError(BaseException):
    """
    The location key is malformatted. This is raised when a primary (non-POI) location key is entered that does not fit
    the formal specifications.
    """

    def __str__(self):
        return(u"Malformatted location key: the location key must be an integer number and must be "
               u"entered as such.")

class NoResultsError(BaseException):
    """
    Yielded when a query has no results.
    """

    def __init__(self, query):
        self.query = query

    def __str__(self):
        return u"Your query for '{0:s}' yielded no results.".format(self.query)


# coding=utf-8
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

description = "Pyccuweather is a simple, efficient back-end to the Accuweather API. Presently, it allows you to " \
              "interface with the API and download forecasts, which it then abstracts into files or passes on to the " \
              "application of your choice. Pyccuweather was designed for integration in large data stores, and as a " \
              "result, it supports export in various numpy/scipy/pandas formats as well."

config = {
    'description': description,
    'author': 'Chris von Csefalvay',
    'url': 'http://github.com/chrisvoncsefalvay/pyccuweather',
    'author_email': 'chris@chrisvoncsefalvay.com',
    'version': '0.31',
    'install_requires': ['nose', 'pandas', 'requests'],
    'packages': ['pyccuweather'],
    'scripts': [],
    'name': 'pyccuweather'
}

setup(requires=['requests', 'nose'], **config)



# coding=utf-8

"""
Pyccuweather
The Python Accuweather API

(c) Chris von Csefalvay, 2015.

http://www.github.com/chrisvoncsefalvay/pyccuweather/
"""

from pyccuweather import errors

ENDPOINT = "http://{api_type:s}.accuweather.com/"
FROOTS = {"loc_geoposition": "locations/v{version:d}/cities/geoposition/search.json",
          "loc_ip_address": "locations/v{version:d}/cities/ipaddress.json",
          "loc_search": "locations/v{version:d}/search.json",
          "loc_search_country": "locations/v{version:d}/{country_code}/search.json",
          "loc_postcode": "locations/v{version:d}/postalcodes/{country_code:s}/search.json",
          "loc_lkey": "locations/v{version:d}/{location_key:d}.json",
          "currentconditions": "currentconditions/v{version:d}/{location_key}.json",
          "currentconditions_6": "currentconditions/v{version:d}/{location_key}/historical.json",
          "currentconditions_24": "currentconditions/v{version:d}/{location_key}/historical/24.json",
          "minutecast_latlon": "forecasts/v{version:d}/minute.json",
          "forecast_1h": "forecasts/v{version:d}/hourly/1hour/{location_key}.json",
          "forecast_12h": "forecasts/v{version:d}/hourly/12hour/{location_key}.json",
          "forecast_24h": "forecasts/v{version:d}/hourly/24hour/{location_key}.json",
          "forecast_72h": "forecasts/v{version:d}/hourly/72hour/{location_key}.json",
          "forecast_120h": "forecasts/v{version:d}/hourly/120hour/{location_key}.json",
          "forecast_240h": "forecasts/v{version:d}/hourly/240hour/{location_key}.json",
          "forecast_1d": "forecasts/v{version:d}/daily/1day/{location_key}.json",
          "forecast_5d": "forecasts/v{version:d}/daily/5day/{location_key}.json",
          "forecast_10d": "forecasts/v{version:d}/daily/10day/{location_key}.json",
          "forecast_15d": "forecasts/v{version:d}/daily/15day/{location_key}.json",
          "forecast_25d": "forecasts/v{version:d}/daily/25day/{location_key}.json",
          "forecast_45d": "forecasts/v{version:d}/daily/45day/{location_key}.json",
          "airquality_current": "airquality/v{version:d}/observations/{location_key}.json",
          "airquality_yesterday": "airquality/v{version:d}/observations/1day/{location_key}.json",
          "climo_actuals_date": "climo/v{version:d}/actuals/{date:s}/{location_key}.json",
          "climo_actuals_range": "climo/v{version:d}/actuals/{location_key}.json",
          "climo_records_date": "climo/v{version:d}/records/{date:s}/{location_key}.json",
          "climo_records_range": "climo/v{version:d}/records/{location_key}.json",
          "climo_normals_date": "climo/v{version:d}/normals/{date:s}/{location_key}.json",
          "climo_normals_range": "climo/v{version:d}/normals/{location_key}.json",
          "climo_month_summary": "climo/v{version:d}/summary/{year:d}/{month:d}/{location_key}.json",
          "alarms_1d": "alarms/v{version:d}/1day/{location_key}",
          "alarms_5d": "alarms/v{version:d}/5day/{location_key}",
          "alarms_10d": "alarms/v{version:d}/10day/{location_key}",
          "alarms_15d": "alarms/v{version:d}/15day/{location_key}",
          "alarms_25d": "alarms/v{version:d}/25day/{location_key}"
          }


def froot(arg, **kwargs):
    """
    Obtains endpoint corresponding to the primary argument and formats it with the keyword arguments provided.

    :param arg: endpoint name
    :param kwargs: endpoint formatting arguments
    :return: endpoint URL suffix
    """
    try:
        assert arg in FROOTS.keys()
    except AssertionError:
        raise errors.NotImplementedOrUnknownMethod(arg)

    try:
        assert "api_type" in kwargs.keys()
    except AssertionError:
        kwargs["api_type"] = "apidev"

    try:
        assert "version" in kwargs.keys()
    except AssertionError:
        kwargs["version"] = 1

    froot_string = ENDPOINT + FROOTS[arg]
    return froot_string.format(**kwargs)
